/******************************************************************************/
/**
 * @file   cmd_bootmode.c
 * @brief  <B>Category</B>Fu-boot<br>
 * @brief  <B>Outline </B>Fu-boot cmd function<br>
 * @date   
 * @author 
 * @note   Note<br>
 *         create new.<br>
 */
/*     */
/******************************************************************************/

#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <fat.h>
#include <mmc.h>
#include <fastboot.h>

#ifdef CONFIG_4460NXSTB

/* size of struct bootloader_message at bootable/recovery/bootloader.h */
#define D_FWUP_BOOTLOADER_MESSAGE_OFFSET    1088

/* alignment 512byte for FWData_t */
#define D_UBOOT_MMC_READSIZE_FWDATA    1536

/* Firmware Data */
typedef struct FWData_t {
    unsigned char boot[D_FWUP_BOOTLOADER_MESSAGE_OFFSET];
    unsigned char SD_flags;
    unsigned char flags;
    unsigned char dummy1;
    unsigned char dummy2;
} __attribute__ ((packed)) FWData_t;

/* Macro and Prototype for secure boot */
#define SECBOOT_FIRST_IMPORTANT_SECTOR_NUM (400 * 1024 / 512)
#define SECBOOT_SKIP_SECTOR_NUM 150
#define SECBOOT_SECTOR_SZ 512

#define SECBOOT_EMMC     0x06
#define SECBOOT_MSG_ADDR 0x81000200
#define SECBOOT_SIG_ADDR 0x81000000
#define SECBOOT_SIG_SZ   256

u32 authenticate_image(u8 *msgAddr, u32 msgSz, u8 *sigAddr, u32 sigSz);
u32 get_boot_device(void);
int get_partition_signed_sector(const char *partname);



/******************************************************************************/
/**
 * @fn     void get_boot_flag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 * @brief  
 * @date   
 * @author 
 * @retval 0:Not boot_flag
           1:boot_flag
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int get_boot_flag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
    int mmcc =1;
    char *ptn = "misc";
    struct fastboot_ptentry *pte;
    char buf[D_UBOOT_MMC_READSIZE_FWDATA];
    FWData_t *fwdata;
    
    pte = fastboot_flash_find_ptn(ptn);
    if (!pte)
    {
        printf("get_boot_flag. err fastboot_flash_find_ptn\n" );
        return 0;
    }

    ret = mmc_init(mmcc);
    if ( ret != 0 )
    {
        printf("get_boot_flag. err mmc_init\n" );
        return 0;
    }

    ret = mmc_read(mmcc, pte->start, (unsigned char *)&buf[0], D_UBOOT_MMC_READSIZE_FWDATA );
    
    if ( ret != 1)
    {
        printf("get_boot_flag. err mmc_read\n" );
        return 0;
    }

    fwdata = &buf[0];
    
    if ( fwdata->flags != 0 )
    {
        ret = 1;
    }
    else
    {
        ret = 0;
    }

    printf("get_boot_flag. flag = %d, ret = %d\n", fwdata->flags, ret );
    return ret;
    
}

U_BOOT_CMD(
        getbootflag,        2,              1,      get_boot_flag,
        "Get Bootflag\n",
        NULL
);

/******************************************************************************/
/**
 * @fn     int set_boot_flag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 * @brief  
 * @date   
 * @author 
 * @retval 0:SUCCESS
           -1:FALSE
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int set_boot_flag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
    int mmcc =1;
    char *ptn = "misc";
    struct fastboot_ptentry *pte;
    char buf[D_UBOOT_MMC_READSIZE_FWDATA];
    FWData_t *fwdata;

    if ( argc != 2 ) 
    {
        printf("set_boot_flag argc err argc = %d\n", argc );
        return -1;
    }
    
    pte = fastboot_flash_find_ptn(ptn);
    if (!pte)
    {
        printf("set_boot_flag. err fastboot_flash_find_ptn\n" );
        return -1;
    }

    ret = mmc_init(mmcc);
    if ( ret != 0 )
    {
        printf("set_boot_flag. err mmc_init\n" );
        return -1;
    }

    ret = mmc_read(mmcc, pte->start, (unsigned char *)&buf[0], D_UBOOT_MMC_READSIZE_FWDATA );
    
    if ( ret != 1)
    {
        printf("set_boot_flag. err mmc_read\n" );
        return -1;
    }

    fwdata = &buf[0];

    fwdata->flags = *argv[1] - 48;
    
    ret = mmc_write(mmcc, (unsigned char *)&buf[0], pte->start, D_UBOOT_MMC_READSIZE_FWDATA );

    if ( ret != 1)
    {
        printf("set_boot_flag. err mmc_write flag = %d\n", fwdata->flags );
        return -1;
    }

    printf("set_boot_flag: write done flag = %d\n", fwdata->flags );

    return 0;
}

U_BOOT_CMD(
        setbootflag,        2,              1,      set_boot_flag,
        "Set Bootflag\n",
        NULL
);

/******************************************************************************/
/**
 * @fn     int set_boot_SDflag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 * @brief  
 * @date   
 * @author 
 * @retval 0:SUCCESS
           -1:FALSE
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int set_boot_SDflag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
    int mmcc =1;
    char *ptn = "misc";
    struct fastboot_ptentry *pte;
    char buf[D_UBOOT_MMC_READSIZE_FWDATA];
    FWData_t *fwdata;

    if ( argc != 2 ) 
    {
        printf("set_boot_SDflag argc err argc = %d\n", argc );
        return -1;
    }
    
    pte = fastboot_flash_find_ptn(ptn);
    if (!pte)
    {
        printf("set_boot_SDflag. err fastboot_flash_find_ptn\n" );
        return -1;
    }

    ret = mmc_init(mmcc);
    if ( ret != 0 )
    {
        printf("set_boot_SDflag. err mmc_init\n" );
        return -1;
    }

    ret = mmc_read(mmcc, pte->start, (unsigned char *)&buf[0], D_UBOOT_MMC_READSIZE_FWDATA );
    
    if ( ret != 1)
    {
        printf("set_boot_SDflag. err mmc_read\n" );
        return -1;
    }

    fwdata = &buf[0];

    fwdata->SD_flags = *argv[1] - 48;

    ret = mmc_write(mmcc, (unsigned char *)&buf[0], pte->start, D_UBOOT_MMC_READSIZE_FWDATA );

    if ( ret != 1)
    {
        printf("set_boot_SDflag. err mmc_write SD_flags = %d\n", fwdata->SD_flags );
        return -1;
    }

    printf("set_boot_SDflag: write done SD_flags = %d\n", fwdata->SD_flags );

    return 0;
}

U_BOOT_CMD(
        setbootsdflag,        2,              1,      set_boot_SDflag,
        "Set BootSDflag\n",
        NULL
);

/******************************************************************************/
/**
 * @fn     void get_boot_SDflag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 * @brief  
 * @date   
 * @author 
 * @retval 0:Not boot_SDflag
           1:boot_SDflag
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int get_boot_SDflag(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
    int mmcc =1;
    char *ptn = "misc";
    struct fastboot_ptentry *pte;
    char buf[D_UBOOT_MMC_READSIZE_FWDATA];
    FWData_t *fwdata;
    
    pte = fastboot_flash_find_ptn(ptn);
    if (!pte)
    {
        printf("get_boot_SDflag. err fastboot_flash_find_ptn\n" );
        return 0;
    }

    ret = mmc_init(mmcc);
    if ( ret != 0 )
    {
        printf("get_boot_SDflag. err mmc_init\n" );
        return 0;
    }

    ret = mmc_read(mmcc, pte->start, (unsigned char *)&buf[0], D_UBOOT_MMC_READSIZE_FWDATA );
    
    if ( ret != 1)
    {
        printf("get_boot_SDflag. err mmc_read\n" );
        return 0;
    }

    fwdata = &buf[0];
    
    if ( fwdata->SD_flags != 0 )
    {
        ret = 1;
    }
    else
    {
        ret = 0;
    }

    printf("get_boot_SDflag. SD_flags = %d, ret = %d\n", fwdata->SD_flags, ret );
    return ret;
    
}

U_BOOT_CMD(
        getbootsdflag,        2,              1,      get_boot_SDflag,
        "Get BootSDflag\n",
        NULL
);

/******************************************************************************/
/**
 * @fn     void get_SDboot_mode(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 * @brief  
 * @date   
 * @author 
 * @retval 0:Not SDboot_mode
           1:SDboot_mode
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int get_SDboot_mode(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
    char buf[512];
    block_dev_desc_t *dev_desc=NULL;
    char *cmd[2];

    if ( argc != 2 ) 
    {
        printf("get_SDboot_mode argc err argc = %d\n", argc );
        return 0;
    }

    /* 1st arg is dev */
    ret = mmc_init(0);
    
    if ( ret != 0 )
    {
        printf("get_SDboot_mode. mmc_init err\n" );
        return 0;
    }

    /* 2nd arg is dev */
    dev_desc = get_dev( "mmc", 0 );
    if ( dev_desc == NULL )
    {
        printf("get_SDboot_mode. get_dev err\n" );
        return 0;
    }

    /* 2nd arg is part */
    ret = fat_register_device( dev_desc, 1 );

    if ( ret != 0 )
    {
        printf("get_SDboot_mode. fat_register_device err\n" );
        return 0;
    }

    ret = file_fat_read( argv[1], &buf[0], 512 );

    if ( ret > 0 )
    {
        printf("get_SDboot_mode file in SD file = %s\n", argv[1] );

        cmd[0] = malloc(20);
        cmd[1] = malloc(10);
        strcpy(cmd[0], "setbootsdflag");
        strcpy(cmd[1], "1");

        ret = set_boot_SDflag(NULL, 0, 2, cmd);

        free(cmd[1]);
        free(cmd[0]);

        if ( ret != 0 )
        {
            printf("get_SDboot_mode. set_boot_SDflag err\n" );
            return 0;
        }

        return 1;
    }
    else
    {
        printf("get_SDboot_mode - No file in SD = %s\n", argv[1] );

        ret = get_boot_SDflag( NULL, 0, argc, argv );

        if ( ret == 0 )
        {
            printf("get_SDboot_mode. No change SDflag\n" );
            return 0;
        }

        cmd[0] = malloc(20);
        cmd[1] = malloc(10);
        strcpy(cmd[0], "setbootsdflag");
        strcpy(cmd[1], "0");

        ret = set_boot_SDflag(NULL, 0, 2, cmd);

        free(cmd[1]);
        free(cmd[0]);

        if ( ret != 0 )
        {
            printf("get_SDboot_mode. set_boot_SDflag err\n" );
            return 0;
        }
        
        return 0;
    }

}

U_BOOT_CMD(
        getSDbootmode,        2,              1,      get_SDboot_mode,
        "Get SDBootMode - read in SD\n",
        NULL
);


/******************************************************************************/
/**
 * @fn     int load_ext4_sparsed_image_from_emmc(char *ptn, unsigned char *dst)
 * @brief  
 * @date   
 * @author 
 * @retval -1:fail
           0<:load size (bytes)
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int load_ext4_sparsed_image_from_emmc(char *ptn, unsigned char *dst)
{
	int mmcc = 1;
	struct fastboot_ptentry *pte;
	int iReadSectorNum = 0;
	int iSignSectorOffset = 0;
	unsigned int uiSparsedStartSector = 0;
	unsigned char *ucpSparsedDst = NULL;
	
	pte = fastboot_flash_find_ptn(ptn);
	if (!pte) {
		printf("cannot find '%s' partition\n", ptn);
		goto fail;
	}
	if (mmc_init(mmcc)) {
		printf("mmc%d init failed\n", mmcc);
		goto fail;
	}
	
	iSignSectorOffset = get_partition_signed_sector(ptn);
	if (iSignSectorOffset < 0) {
		printf("cannot find '%s' partition sign\n", ptn);
		goto fail;
	}
	
	if (mmc_read(mmcc, pte->start, dst,
								SECBOOT_FIRST_IMPORTANT_SECTOR_NUM * SECBOOT_SECTOR_SZ)
				!= 1) {
		printf("mmc failed to read image\n");
		goto fail;
	}
	
	ucpSparsedDst = dst + SECBOOT_FIRST_IMPORTANT_SECTOR_NUM * SECBOOT_SECTOR_SZ;
	iSignSectorOffset = iSignSectorOffset - SECBOOT_FIRST_IMPORTANT_SECTOR_NUM;
	uiSparsedStartSector = pte->start + SECBOOT_FIRST_IMPORTANT_SECTOR_NUM;
	
	for(iReadSectorNum = 0;
			iReadSectorNum < (iSignSectorOffset + (SECBOOT_SKIP_SECTOR_NUM - 1)) / SECBOOT_SKIP_SECTOR_NUM;
			iReadSectorNum++){
		if (mmc_read(mmcc, uiSparsedStartSector + iReadSectorNum * SECBOOT_SKIP_SECTOR_NUM,
									ucpSparsedDst + iReadSectorNum * SECBOOT_SECTOR_SZ, SECBOOT_SECTOR_SZ)
					!= 1) {
			printf("mmc failed to read image\n");
			goto fail;
		}
	}
	
	
	return (SECBOOT_FIRST_IMPORTANT_SECTOR_NUM + iReadSectorNum) * SECBOOT_SECTOR_SZ;

fail:
	return -1;
}



/******************************************************************************/
/**
 * @fn     int load_sparsed_image_from_emmc(char *ptn, unsigned char *dst)
 * @brief  
 * @date   
 * @author 
 * @retval -1:fail
           0<:load size (bytes)
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int load_sparsed_image_from_emmc(char *ptn, unsigned char *dst)
{
	int mmcc = 1;
	struct fastboot_ptentry *pte;
	int iReadSectorNum = 0;
	int iSignSectorOffset = 0;
	
	pte = fastboot_flash_find_ptn(ptn);
	if (!pte) {
		printf("cannot find '%s' partition\n", ptn);
		goto fail;
	}
	if (mmc_init(mmcc)) {
		printf("mmc%d init failed\n", mmcc);
		goto fail;
	}
	
	iSignSectorOffset = get_partition_signed_sector(ptn);
	if (iSignSectorOffset < 0) {
		printf("cannot find '%s' partition sign\n", ptn);
		goto fail;
	}
	
	for(iReadSectorNum = 0;
			iReadSectorNum < (iSignSectorOffset + (SECBOOT_SKIP_SECTOR_NUM - 1)) / SECBOOT_SKIP_SECTOR_NUM;
			iReadSectorNum++){
		if (mmc_read(mmcc, pte->start + iReadSectorNum * SECBOOT_SKIP_SECTOR_NUM,
									dst + iReadSectorNum * SECBOOT_SECTOR_SZ, SECBOOT_SECTOR_SZ)
					!= 1) {
			printf("mmc failed to read image\n");
			goto fail;
		}
	}
	
	
	return iReadSectorNum * SECBOOT_SECTOR_SZ;

fail:
	return -1;
}


/******************************************************************************/
/**
 * @fn     int load_sign_from_emmc(char *ptn, unsigned char *dst)
 * @brief  
 * @date   
 * @author 
 * @retval -1:fail
           0<:load size (bytes)
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
int load_sign_from_emmc(char *ptn, unsigned char *dst)
{
	int mmcc = 1;
	struct fastboot_ptentry *pte;
	int iSignSectorOffset = 0;
	
	pte = fastboot_flash_find_ptn(ptn);
	if (!pte) {
		printf("cannot find '%s' partition\n", ptn);
		goto fail;
	}
	
	iSignSectorOffset = get_partition_signed_sector(ptn);
	if (iSignSectorOffset < 0) {
		printf("cannot find '%s' partition sign\n", ptn);
		goto fail;
	}
	
	if (mmc_read(mmcc, pte->start + iSignSectorOffset, dst, SECBOOT_SECTOR_SZ) != 1) {
		printf("mmc failed to read image\n");
		goto fail;
  }
	
	return 0;

fail:
	return -1;
}

/******************************************************************************/
/**
 * @fn     void boot_recovery(void)
 * @brief  
 * @date   
 * @author 
 * @retval 
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
void boot_recovery(void)
{
    char *cmd[3];
    
#ifdef SECBOOT_ENABLE
    int iReadSz = 0;
    
    if(get_boot_device() == SECBOOT_EMMC){
        /* If boot device is EMMC, doing secure boot. */
        /* Authentication recovery partition */
        if(load_sign_from_emmc("recovery", (u8 *)SECBOOT_SIG_ADDR) != 0){
            printf("SECUREBOOT: recovery part sign reading failed.\n");
#ifdef SECBOOT_ENABLE_HANG
            hang();
#endif /* SECBOOT_ENABLE_HANG */
        }
        else{
            iReadSz = load_sparsed_image_from_emmc("recovery",
                                                   (u8 *)SECBOOT_MSG_ADDR);
            if(iReadSz <= 0){
                printf("SECUREBOOT: recovery part msg reading failed. "
                       "iReadSz=%d\n", iReadSz);
#ifdef SECBOOT_ENABLE_HANG
                hang();
#endif /* SECBOOT_ENABLE_HANG */
            }
            else{
                if(authenticate_image((u8 *)SECBOOT_MSG_ADDR, iReadSz,
                                      (u8 *)SECBOOT_SIG_ADDR, SECBOOT_SIG_SZ)
                   != 0){
                    printf("SECUREBOOT: recovery part authentication failed.\n");
#ifdef SECBOOT_ENABLE_HANG
                    hang();
#endif /* SECBOOT_ENABLE_HANG */
                }
                else{
                    /* If Secure Boot is successed, do nothing */
                    printf("SECUREBOOT: recovery part authenticated.\n");
                }
            }
        }
    }
#endif /* SECBOOT_ENABLE */
    
    cmd[0] = malloc(10);
    cmd[1] = malloc(10);
    cmd[2] = malloc(10);

    strcpy(cmd[0], "booti");
    strcpy(cmd[1], "mmc1");
    strcpy(cmd[2], "recovery");

    do_booti(NULL, 0, 3, cmd);

    free(cmd[2]);
    free(cmd[1]);
    free(cmd[0]);

    return;
}

/******************************************************************************/
/**
 * @fn     void boot_normal(void)
 * @brief  
 * @date   
 * @author 
 * @retval 
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
void boot_normal(void)
{
    char *cmd[2];
    
#ifdef SECBOOT_ENABLE
    int iReadSz = 0;
    
    if(get_boot_device() == SECBOOT_EMMC){
        /* If boot device is EMMC, doing secure boot. */
        if(load_sign_from_emmc("boot", (u8 *)SECBOOT_SIG_ADDR) != 0){
            printf("SECUREBOOT: boot part sign reading failed.\n");
#ifdef SECBOOT_ENABLE_HANG
            hang();
#endif /* SECBOOT_ENABLE_HANG */
        }
        else{
            iReadSz = load_sparsed_image_from_emmc("boot", (u8 *)SECBOOT_MSG_ADDR);
            if(iReadSz <= 0){
                printf("SECUREBOOT: boot part msg reading failed. iReadSz=%d\n", iReadSz);
#ifdef SECBOOT_ENABLE_HANG
                hang();
#endif /* SECBOOT_ENABLE_HANG */
            }
            else{
                if(authenticate_image((u8 *)SECBOOT_MSG_ADDR, iReadSz,
                                      (u8 *)SECBOOT_SIG_ADDR, SECBOOT_SIG_SZ)
                   != 0){
                    printf("SECUREBOOT: boot part authentication failed.\n");
#ifdef SECBOOT_ENABLE_HANG
                    hang();
#endif /* SECBOOT_ENABLE_HANG */
                }
                else{
                    printf("SECUREBOOT: boot part authenticated.\n");
                }
            }
        }
        
        if(load_sign_from_emmc("system", (u8 *)SECBOOT_SIG_ADDR) != 0){
            printf("SECUREBOOT: system part sign reading failed. iReadSz=%d\n", iReadSz);
#ifdef SECBOOT_ENABLE_HANG
            hang();
#endif /* SECBOOT_ENABLE_HANG */
        }
        else{
            iReadSz = load_ext4_sparsed_image_from_emmc("system", (u8 *)SECBOOT_MSG_ADDR);
            if(iReadSz <= 0){
                printf("SECUREBOOT: system part msg reading failed.\n");
#ifdef SECBOOT_ENABLE_HANG
                hang();
#endif /* SECBOOT_ENABLE_HANG */
            }
            else{
                if(authenticate_image((u8 *)SECBOOT_MSG_ADDR, iReadSz,
                                      (u8 *)SECBOOT_SIG_ADDR, SECBOOT_SIG_SZ)
                   != 0){
                    printf("SECUREBOOT: system part authentication failed.\n");
#ifdef SECBOOT_ENABLE_HANG
                    hang();
#endif /* SECBOOT_ENABLE_HANG */
                }
                else{
                    printf("SECUREBOOT: system part authenticated.\n");
                }
            }
        }
    }
#endif /* SECBOOT_ENABLE */
    
    cmd[0] = malloc(10);
    cmd[1] = malloc(10);

    strcpy(cmd[0], "booti");
    strcpy(cmd[1], "mmc1");

    do_booti(NULL, 0, 2, cmd);

    free(cmd[1]);
    free(cmd[0]);

    return;
}


/******************************************************************************/
/**
 * @fn     void select_boot (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
 * @brief  
 * @date   
 * @author 
 * @retval 
 * @note   Note<br>
 *         create new.<br>
*/
/******************************************************************************/
void select_boot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int ret;
 
    ret = get_SDboot_mode( NULL, 0, argc, argv );
    
    if( ret == 1 )
    {
        printf("select_boot recovery SD\n" );
        boot_recovery();
        return;
    }

    ret = get_boot_flag( NULL, 0, argc, argv );

    if( ret == 1 )
    {
        printf("select_boot recovery EMMC\n" );
        boot_recovery();
        return;
    }

    printf("select_boot normal\n" );
    boot_normal();
    return;

}

U_BOOT_CMD(
        selectboot,        2,              1,      select_boot,
        "Select Boot\n",
        NULL
);

#else /* CONFIG_4460NXSTB */
#endif /* CONFIG_4460NXSTB */
