/*
 * (C) Copyright 2004-2009
 * Texas Instruments, <www.ti.com>
 * Richard Woodruff <r-woodruff2@ti.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#include <asm/arch/cpu.h>
#include <asm/io.h>
#include <asm/arch/bits.h>
#include <asm/arch/mux.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/arch/clocks.h>
#include <asm/arch/mem.h>
#include <i2c.h>
#include <twl6030.h>
#include <asm/mach-types.h>
#if (CONFIG_COMMANDS & CFG_CMD_NAND) && defined(CFG_NAND_LEGACY)
#include <linux/mtd/nand_legacy.h>
#endif

#ifdef CONFIG_4460NXSTB
void led_init( void );
#endif /* CONFIG_4460NXSTB */

/*******************************************************
 * Routine: delay
 * Description: spinning delay to use before udelay works
 ******************************************************/
static inline void delay(unsigned long loops)
{
	__asm__ volatile ("1:\n" "subs %0, %1, #1\n"
			  "bne 1b" : "=r" (loops) : "0"(loops));
}

/*****************************************
 * Routine: secure_unlock
 * Description: Setup security registers for access
 * (GP Device only)
 *****************************************/
void secure_unlock_mem(void)
{
	/* Permission values for registers -Full fledged permissions to all */
	#define UNLOCK_1 0xFFFFFFFF
	#define UNLOCK_2 0x00000000
	#define UNLOCK_3 0x0000FFFF

	/* Protection Module Register Target APE (PM_RT)*/
	__raw_writel(UNLOCK_1, RT_REQ_INFO_PERMISSION_1);
	__raw_writel(UNLOCK_1, RT_READ_PERMISSION_0);
	__raw_writel(UNLOCK_1, RT_WRITE_PERMISSION_0);
	__raw_writel(UNLOCK_2, RT_ADDR_MATCH_1);

	__raw_writel(UNLOCK_3, GPMC_REQ_INFO_PERMISSION_0);
	__raw_writel(UNLOCK_3, GPMC_READ_PERMISSION_0);
	__raw_writel(UNLOCK_3, GPMC_WRITE_PERMISSION_0);

	__raw_writel(UNLOCK_3, OCM_REQ_INFO_PERMISSION_0);
	__raw_writel(UNLOCK_3, OCM_READ_PERMISSION_0);
	__raw_writel(UNLOCK_3, OCM_WRITE_PERMISSION_0);
	__raw_writel(UNLOCK_2, OCM_ADDR_MATCH_2);

	/* IVA Changes */
	__raw_writel(UNLOCK_3, IVA2_REQ_INFO_PERMISSION_0);
	__raw_writel(UNLOCK_3, IVA2_READ_PERMISSION_0);
	__raw_writel(UNLOCK_3, IVA2_WRITE_PERMISSION_0);

	__raw_writel(UNLOCK_3, IVA2_REQ_INFO_PERMISSION_1);
	__raw_writel(UNLOCK_3, IVA2_READ_PERMISSION_1);
	__raw_writel(UNLOCK_3, IVA2_WRITE_PERMISSION_1);

	__raw_writel(UNLOCK_3, IVA2_REQ_INFO_PERMISSION_2);
	__raw_writel(UNLOCK_3, IVA2_READ_PERMISSION_2);
	__raw_writel(UNLOCK_3, IVA2_WRITE_PERMISSION_2);

	__raw_writel(UNLOCK_3, IVA2_REQ_INFO_PERMISSION_3);
	__raw_writel(UNLOCK_3, IVA2_READ_PERMISSION_3);
	__raw_writel(UNLOCK_3, IVA2_WRITE_PERMISSION_3);

	__raw_writel(UNLOCK_1, SMS_RG_ATT0); /* SDRC region 0 public */
}


/**********************************************************
 * Routine: secureworld_exit()
 * Description: If chip is EMU and boot type is external
 *		configure secure registers and exit secure world
 *  general use.
 ***********************************************************/

void secureworld_exit(void)
{
	unsigned long i;

	/* configrue non-secure access control register */
	__asm__ __volatile__("mrc p15, 0, %0, c1, c1, 2" : "=r" (i));
	/* enabling co-processor CP10 and CP11 accesses in NS world */
	__asm__ __volatile__("orr %0, %0, #0xC00" : "=r"(i));
	/* allow allocation of locked TLBs and L2 lines in NS world */
	/* allow use of PLE registers in NS world also */
	__asm__ __volatile__("orr %0, %0, #0x70000" : "=r"(i));
	__asm__ __volatile__("mcr p15, 0, %0, c1, c1, 2" : "=r" (i));

	/*
	 * Enable IBE in ACR register.ASA is disabled following
	 * recommendation from ARM
	 */
	__asm__ __volatile__("mrc p15, 0, %0, c1, c0, 1" : "=r" (i));
	__asm__ __volatile__("orr %0, %0, #0x40" : "=r"(i));
	__asm__ __volatile__("mcr p15, 0, %0, c1, c0, 1" : "=r" (i));

	/* Exiting secure world */
	__asm__ __volatile__("mrc p15, 0, %0, c1, c1, 0" : "=r" (i));
	__asm__ __volatile__("orr %0, %0, #0x31" : "=r"(i));
	__asm__ __volatile__("mcr p15, 0, %0, c1, c1, 0" : "=r" (i));
}

/**********************************************************
 * Routine: try_unlock_sram()
 * Description: If chip is GP/EMU(special) type, unlock the SRAM for
 *  general use.
 ***********************************************************/
int get_boot_type(void);
void v7_flush_dcache_all(int , int);
void setup_auxcr(int , int);

void try_unlock_memory(void)
{
	int mode;
	int in_sdram = running_in_sdram();

	/* if GP device unlock device SRAM for general use */
	/* secure code breaks for Secure/Emulation device - HS/E/T*/
	mode = get_device_type();
	if (mode == GP_DEVICE)
		secure_unlock_mem();

	/* If device is EMU and boot is XIP external booting
	 * Unlock firewalls and disable L2 and put chip
	 * out of secure world
	 */
	/* Assuming memories are unlocked by the demon who put us in SDRAM */
	if ((mode <= EMU_DEVICE) && (get_boot_type() == 0x1F)
		&& (!in_sdram)) {
		secure_unlock_mem();
		secureworld_exit();
	}

	return;
}

/**********************************************************
 * Routine: s_init
 * Description: Does early system init of muxing and clocks.
 * - Called path is with SRAM stack.
 **********************************************************/

void s_init(void)
{
	int external_boot = 0;
	int in_sdram = running_in_sdram();

#ifdef CONFIG_4430VIRTIO
	in_sdram = 0;  /* allow setup from memory for Virtio */
#endif
	/* TODO: Disable Watchdog's here if needed. */
	watchdog_init();
	
#if 0
	external_boot = (get_boot_type() == 0x1F) ? 1 : 0;
	/* Right now flushing at low MPU speed. Need to move after clock init */
	v7_flush_dcache_all(get_device_type(), external_boot);

	try_unlock_memory();
#endif
#ifndef CONFIG_ICACHE_OFF
	icache_enable();
#endif
	/* Clock and Mux configerations is already done in x-loader */
	if(!in_sdram) {
		set_muxconf_regs();
		delay(100);
		prcm_init();
	}
}

/*******************************************************
 * Routine: misc_init_r
 * Description: Init ethernet (done here so udelay works)
 ********************************************************/
int misc_init_r(void)
{
#ifdef CONFIG_DRIVER_OMAP44XX_I2C
	i2c_init(CFG_I2C_SPEED, CFG_I2C_SLAVE);
#ifdef CONFIG_TWL6030
	twl6030_init_battery_charging();
#endif
#ifdef CONFIG_4460NXSTB
    led_init( );
#endif /* CONFIG_4460NXSTB */
#endif
	return 0;
}

/******************************************************
 * Routine: wait_for_command_complete
 * Description: Wait for posting to finish on watchdog
 ******************************************************/
void wait_for_command_complete(unsigned int wd_base)
{
	int pending = 1;
	do {
		pending = __raw_readl(wd_base + WWPS);
	} while (pending);
}

/****************************************
 * Routine: watchdog_init
 * Description: Shut down watch dogs
 *****************************************/
void watchdog_init(void)
{
	/* There are 3 watch dogs WD1=Secure, WD2=MPU, WD3=IVA. WD1 is
	 * either taken care of by ROM (HS/EMU) or not accessible (GP).
	 * We need to take care of WD2-MPU or take a PRCM reset.  WD3
	 * should not be running and does not generate a PRCM reset.
	 */
	
	__raw_writel(WD_UNLOCK1, WD2_BASE + WSPR);
	wait_for_command_complete(WD2_BASE);
	__raw_writel(WD_UNLOCK2, WD2_BASE + WSPR);
}

/*******************************************************************
 * Routine:ether_init
 * Description: take the Ethernet controller out of reset and wait
 *  		   for the EEPROM load to complete.
 ******************************************************************/
void ether_init(void)
{
#ifdef CONFIG_DRIVER_LAN91C96
	int cnt = 20;

	__raw_writew(0x0, LAN_RESET_REGISTER);
	do {
		__raw_writew(0x1, LAN_RESET_REGISTER);
		udelay(100);
		if (cnt == 0)
			goto h4reset_err_out;
		--cnt;
	} while (__raw_readw(LAN_RESET_REGISTER) != 0x1);

	cnt = 20;

	do {
		__raw_writew(0x0, LAN_RESET_REGISTER);
		udelay(100);
		if (cnt == 0)
			goto h4reset_err_out;
		--cnt;
	} while (__raw_readw(LAN_RESET_REGISTER) != 0x0000);
	udelay(1000);

	*((volatile unsigned char *)ETH_CONTROL_REG) &= ~0x01;
	udelay(1000);

       h4reset_err_out:
	return;
#endif
}

/*
 * This function finds the SDRAM size available in the system
 * based on DMM section configurations
 * This is needed because the size of memory installed may be
 * different on different versions of the board
 */
u32 sdram_size(void)
{
	u32 section, i, total_size = 0, size, addr;
	u32 sdrc_addrspc;
	u32 sdrc_map;

	for (i = 0; i < 4; i++) {
		section	= __raw_readl(DMM_LISA_MAP + i*4);
		addr = section & DMM_LISA_MAP_SYS_ADDR_MASK;
		sdrc_addrspc = (section & DMM_LISA_MAP_SDRC_ADDRSPC_MASK) >>
			DMM_LISA_MAP_SDRC_ADDRSPC_SHIFT;
		sdrc_map = (section & DMM_LISA_MAP_SDRC_MAP_MASK) >>
			DMM_LISA_MAP_SDRC_MAP_SHIFT;

		/* Take into account only mapped sections */
		if (0 == sdrc_map)
			continue;

		/* Handle only DMM sections which define physical SDRAM
		 * configuration */
		if (DMM_SDRC_ADDRSPC_SDRAM != sdrc_addrspc)
			continue;

		/* update SDRAM size */
		size	= ((section & DMM_LISA_MAP_SYS_SIZE_MASK) >>
			    DMM_LISA_MAP_SYS_SIZE_SHIFT);
		size	= 1 << size;
		size	*= SZ_16M;
		total_size += size;

	}
	return total_size;
}

/**********************************************
 * Routine: dram_init
 * Description: sets uboots idea of sdram size
 **********************************************/
int dram_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	u32 mtype, btype;

	btype = get_board_type();
	mtype = get_mem_type();
	/* fixme... dont know why this func is crashing in ZeBu */
	//display_board_info(btype);

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = sdram_size();

	printf("Load address: 0x%x\n", TEXT_BASE);
	return 0;
}

#if (CONFIG_COMMANDS & CFG_CMD_NAND) && defined(CFG_NAND_LEGACY)
/**********************************************************
 * Routine: nand+_init
 * Description: Set up nand for nand and jffs2 commands
 *********************************************************/
void nand_init(void)
{
	/* REVISIT */
	return;
}
#endif

#ifdef CONFIG_4460NXSTB
/**********************************************************
 * Routine: led_init
 * Description: led Initialzation
 *********************************************************/
void led_init( void )
{
    unsigned char   slaveaddr;
    unsigned int    addr;
    unsigned char   data;
    int             ret;

    /* select I2C3 */
    select_bus( 2, CFG_I2C_SPEED );

    /* Detect LED Device  */
    slaveaddr   = 0x62;
    addr        = 0x02;
    data        = 0;
    /* NXP LED RED Reg Access */
    udelay(2000);
    ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
    if( 0 != ret){ /* LED Device of SANYO */
        slaveaddr   = 0x75;                /* LED Controller */
        data        = 0;                   /* LED SW Off     */
        addr = 0x00;                       /* LED-R          */
        ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
        if( 0 != ret )
        {
            printf( "%s write ERR addr = 0x%x data = 0x%x \n", __FUNCTION__, addr, data );
        }
        addr = 0x01;                       /* LED-G          */
        ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
        if( 0 != ret )
        {
            printf( "%s write ERR addr = 0x%x data = 0x%x \n", __FUNCTION__, addr, data );
        }
        addr = 0x02;                       /* LED-B          */
        ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
        if( 0 != ret )
        {
            printf( "%s write ERR addr = 0x%x data = 0x%x \n", __FUNCTION__, addr, data );
        }
    }
    else{ /* LED Device of NXP */
        slaveaddr   = 0x03; /* LED NXP Controller      */
        addr        = 0xA5; /* Software Reset          */
        data        = 0x5A; /* Init Date               */
        ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
        if( 0 != ret )
        {
            printf( "%s write ERR addr = 0x%x data = 0x%x \n", __FUNCTION__, addr, data );
        }
        udelay(5);
        slaveaddr   = 0x62; /* LED NXP Controller      */
        addr        = 0x00; /* Mode1                   */
        data        = 0x01; /* Init Date               */
        ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
        if( 0 != ret )
        {
            printf( "%s write ERR addr = 0x%x data = 0x%x \n", __FUNCTION__, addr, data );
        }
        slaveaddr   = 0x62; /* LED NXP Controller      */
        addr        = 0x01; /* Mode2                   */
        data        = 0x01; /* Init Date               */
        ret = i2c_write( slaveaddr, addr, 1, &data, 1 );
        if( 0 != ret )
        {
            printf( "%s write ERR addr = 0x%x data = 0x%x \n", __FUNCTION__, addr, data );
        }
    }

    /* select I2C1 */
    select_bus( 0, CFG_I2C_SPEED );

    return;
}
#endif /* CONFIG_4460NXSTB */
