#ifndef __SPI__H__
#define __SPI__H__

#define MCSPI_SPI4_IO_BASE	0x480BA110
#define MCSPI_SPI3_IO_BASE	0x480B8110
#define MCSPI_SPI2_IO_BASE	0x4809A110
#define MCSPI_SPI1_IO_BASE	0x48098110
#define TXS			0
#define RXS			1


#define MIO_DWORD(x)  *((volatile unsigned long* )( x ))
#define MIO_WORD(x)   *((volatile unsigned short* )( x ))
#define MIO_BYTE(x)   *((volatile unsigned char* )( x ))

/*
 * Bitfield macros that use rely on bitfield width/shift information
 * defined in CSP header files
 */
#define CSP_BITFMASK(bit) (((1U << (bit ## _WID)) - 1) << (bit ## _LSH))
#define CSP_BITFVAL(bit, val) (((val) & ((1U << (bit ## _WID)) - 1)) << (bit ## _LSH))

/* Create a maximum value with a bitfield width */
#define BIT_MAX(bf)  ((1U << (bf##_WID)) - 1)

/*
 * Macros for bitfield operations on data variables. DO NOT use for peripheral
 * register accesses.  Use the INREG/OUTREG based macros above for peripheral
 * accesses
 */
#define CSP_BITFEXT(var, bit) ((var & CSP_BITFMASK(bit)) >> (bit ## _LSH))
#define CSP_BITFCLR(var, bit) (var &= (~CSP_BITFMASK(bit)))
#define CSP_BITFINS(var, bit, val) \
    (CSP_BITFCLR(var, bit)); (var |= CSP_BITFVAL(bit, val))

/* Macros for generating 64-bit IRQ masks */
#define CSP_IRQMASK(irq) (((ULONGLONG) 1)  << irq)

/*
 * Multipad for spi1
 */

#define CONTROL_PADCONF_MCSPI1_CLK	0x4A100130
#define CONTROL_PADCONF_MCSPI1_SOMI	0x4A100134
#define CONTROL_PADCONF_MCSPI1_CS0	0x4A100138

#define CONTROL_PADCONF_MUXMODE1_LSH	16
#define CONTROL_PADCONF_MUXMODE1_WID    3

#define CONTROL_PADCONF_MUXMODE0_LSH	0x0
#define CONTROL_PADCONF_MUXMODE0_WID    3

#define CM_FCLKEN1_CORE_BASE		0x48004A00
#define CM_ICLKEN1_CORE_BASE		0x48004A10

#define SPI1_ENABLE_CLOCK_FUNCTION	18
#define SPI2_ENABLE_CLOCK_FUNCTION	19
#define SPI3_ENABLE_CLOCK_FUNCTION	20
#define SPI4_ENABLE_CLOCK_FUNCTION	21

#define MCSPI_XFERLEVEL			0x4809817C

typedef unsigned long REG32;

typedef struct _mscpi_regs {
REG32		SYSCONFIG;
REG32		SYSSTATUS;
REG32		IRQSTATUS;
REG32		IRQENABLE;
REG32		WAKEUPENABLE;
REG32		SYST;
REG32		MODULECTRL;
REG32		CHxCONF;
REG32		CHxSTAT;
REG32		CHxCTRL;
REG32		TXx;
REG32		RXx;
unsigned char reserved[64];
REG32   XFERLEVEL
} MCSPI_REGS;

/* MCSPI_SYSCONFIG */
#define SPI_SYSCONFIG_AUTOIDLE_LSH		0
#define SPI_SYSCONFIG_AUTOIDLE_WID		1
#define SPI_SYSCONFIG_SOFTRESET_LSH		1
#define SPI_SYSCONFIG_SOFTRESET_WID		1
#define SPI_SYSCONFIG_ENAWAKEUP_LSH		2
#define SPI_SYSCONFIG_ENAWAKEUP_WID		1
#define SPI_SYSCONFIG_SIDLEMODE_LSH		3
#define SPI_SYSCONFIG_SIDLEMODE_WID		2
#define SPI_SYSCONFIG_CLOCKACTIVITY_LSH		8
#define SPI_SYSCONFIG_CLOCKACTIVITY_WID		2


/* MCSPI_SYSSTATUS */
#define SPI_SYSSTATUS_RESETDONE_LSH		0
#define SPI_SYSSTATUS_RESETDONE_WID		1

/* MCSPI_IRQSTATUS */
#define SPI_SYSSTATUS_TX0EMPTY_LSH		0
#define SPI_SYSSTATUS_TX0EMPTY_WID		1
#define SPI_SYSSTATUS_TX0UNDERFLOW_LSH		1
#define SPI_SYSSTATUS_TX0UNDERFLOW_WID		1
#define SPI_SYSSTATUS_RX0FULL_LSH		2
#define SPI_SYSSTATUS_RX0FULL_WID		1
#define SPI_SYSSTATUS_RX0OVERFLOW_LSH		3
#define SPI_SYSSTATUS_RX0OVERFLOW_WID		1
#define SPI_SYSSTATUS_TX1EMPTY_LSH		4
#define SPI_SYSSTATUS_TX1EMPTY_WID		1
#define SPI_SYSSTATUS_TX1UNDERFLOW_LSH		5
#define SPI_SYSSTATUS_TX1UNDERFLOW_WID		1
#define SPI_SYSSTATUS_RX1FULL_LSH		6
#define SPI_SYSSTATUS_RX1FULL_WID		1
#define SPI_SYSSTATUS_TX2EMPTY_LSH		8
#define SPI_SYSSTATUS_TX2EMPTY_WID		1
#define SPI_SYSSTATUS_TX2UNDERFLOW_LSH		9
#define SPI_SYSSTATUS_TX2UNDERFLOW_WID		1
#define SPI_SYSSTATUS_RX2FULL_LSH		10
#define SPI_SYSSTATUS_RX2FULL_WID		1
#define SPI_SYSSTATUS_TX3EMPTY_LSH		12
#define SPI_SYSSTATUS_TX3EMPTY_WID		1
#define SPI_SYSSTATUS_TX3UNDERFLOW_LSH		13
#define SPI_SYSSTATUS_TX2UNDERFLOW_WID		1
#define SPI_SYSSTATUS_RX3FULL_LSH		14
#define SPI_SYSSTATUS_RX2FULL_WID		1
#define SPI_SYSSTATUS_WKS_LSH			16
#define SPI_SYSSTATUS_WKS_WID			1
#define SPI_SYSSTATUS_EOW_LSH			17
#define SPI_SYSSTATUS_EOW_WID			1

/* MCSPI_IRQENABLE */
#define SPI_IRQENABLE_TX0EMPTY_LSH		0
#define SPI_IRQENABLE_TX0EMPTY_WID		1
#define SPI_IRQENABLE_TX0UNDERFLOW_LSH		1
#define SPI_IRQENABLE_TX0UNDERFLOW_WID		1
#define SPI_IRQENABLE_RX0FULL_LSH		2
#define SPI_IRQENABLE_RX0FULL_WID		1
#define SPI_IRQENABLE_RX0OVERFLOW_LSH		3
#define SPI_IRQENABLE_RX0OVERFLOW_WID		1
#define SPI_IRQENABLE_TX1EMPTY_LSH		4
#define SPI_IRQENABLE_TX1EMPTY_WID		1
#define SPI_IRQENABLE_TX1UNDERFLOW_LSH		5
#define SPI_IRQENABLE_TX1UNDERFLOW_WID		1
#define SPI_IRQENABLE_RX1FULL_LSH		6
#define SPI_IRQENABLE_RX1FULL_WID		1
#define SPI_IRQENABLE_TX2EMPTY_LSH		8
#define SPI_IRQENABLE_TX2EMPTY_WID		1
#define SPI_IRQENABLE_TX2UNDERFLOW_LSH		9
#define SPI_IRQENABLE_TX2UNDERFLOW_WID		1
#define SPI_IRQENABLE_RX2FULL_LSH		10
#define SPI_IRQENABLE_RX2FULL_WID		1
#define SPI_IRQENABLE_TX3EMPTY_LSH		12
#define SPI_IRQENABLE_TX3EMPTY_WID		1
#define SPI_IRQENABLE_TX3UNDERFLOW_LSH		13
#define SPI_IRQENABLE_TX2UNDERFLOW_WID		1
#define SPI_IRQENABLE_RX3FULL_LSH		14
#define SPI_IRQENABLE_RX2FULL_WID		1
#define SPI_IRQENABLE_WKE_LSH			16
#define SPI_IRQENABLE_WKE_WID			1
#define SPI_IRQENABLE_EOWKE_LSH			17
#define SPI_IRQENABLE_EOWKE_WID			1

/* MCSPI_WAKEUPENABLE */
#define SPI_WAKEUPENABLE_WKEN_LSH		0
#define SPI_WAKEUPENABLE_WKEN_WID		1

/* MCSPI_SYST */
#define SPI_SYST_SPIEN0_LSH			0
#define SPI_SYST_SPIEN0_WID			1
#define SPI_SYST_SPIEN1_LSH			1
#define SPI_SYST_SPIEN1_WID			1
#define SPI_SYST_PIEN2_LSH			2
#define SPI_SYST_PIEN2_WID			1
#define SPI_SYST_SPIEN3_LSH			3
#define SPI_SYST_SPIEN3_WID			1
#define SPI_SYST_SPIDAT0_LSH			4
#define SPI_SYST_SPIDAT0_WID			1
#define SPI_SYST_SPIDAT1_LSH			5
#define SPI_SYST_SPIDAT1_WID			1
#define SPI_SYST_SPICLK_LSH			6
#define SPI_SYST_SPICLK_WID			1
#define SPI_SYST_WAKD_LSH			7
#define SPI_SYST_WAKD_WID			1
#define SPI_SYST_SPIDATDIR0_LSH			8
#define SPI_SYST_SPIDATDIR0_WID			1
#define SPI_SYST_SPIDATDIR1_LSH			9
#define SPI_SYST_SPIDATDIR1_WID			1
#define SPI_SYST_SPIENDIR_LSH			10
#define SPI_SYST_SPIENDIR_WID			1
#define SPI_SYST_SSB_LSH			11
#define SPI_SYST_SSB_WID			1

/* MCSPI_MODULCTRL */
#define SPI_MODULCTRL_SINGLE_LSH		0
#define SPI_MODULCTRL_SINGLE_WID		1
#define SPI_MODULCTRL_MS_LSH			2
#define SPI_MODULCTRL_MS_WID			1
#define SPI_MODULCTRL_SYSTEST_LSH		3
#define SPI_MODULCTRL_SYSTEST_WID		1

/* MCSPI_CHxCONF */
#define SPI_CHxCONF_PHA_LSH			0
#define SPI_CHxCONF_PHA_WID			1
#define SPI_CHxCONF_POL_LSH			1
#define SPI_CHxCONF_POL_WID			1
#define SPI_CHxCONF_CLKD_LSH			2
#define SPI_CHxCONF_CLKD_WID			4
#define SPI_CHxCONF_EPOL_LSH			6
#define SPI_CHxCONF_EPOL_WID			1
#define SPI_CHxCONF_WL_LSH			7
#define SPI_CHxCONF_WL_WID			5
#define SPI_CHxCONF_TRM_LSH			12
#define SPI_CHxCONF_TRM_WID			2
#define SPI_CHxCONF_DMAW_LSH			14
#define SPI_CHxCONF_DMAW_WID			1
#define SPI_CHxCONF_DMAR_LSH			15
#define SPI_CHxCONF_DMAR_WID			1
#define SPI_CHxCONF_DPE0_LSH			16
#define SPI_CHxCONF_DPE0_WID			1
#define SPI_CHxCONF_DPE1_LSH			17
#define SPI_CHxCONF_DPE1_WID			1
#define SPI_CHxCONF_IS_LSH			18
#define SPI_CHxCONF_IS_WID			1
#define SPI_CHxCONF_TURBO_LSH			19
#define SPI_CHxCONF_TURBO_WID			1
#define SPI_CHxCONF_FORCE_LSH			20
#define SPI_CHxCONF_FORCE_WID			1
#define SPI_CHxCONF_SPIENSLV_LSH		21
#define SPI_CHxCONF_SPIENSLV_WID		2
#define SPI_CHxCONF_SBE_LSH			23
#define SPI_CHxCONF_SBE_WID			1
#define SPI_CHxCONF_SBPOL_LSH			24
#define SPI_CHxCONF_SBPOL_WID			1
#define SPI_CHxCONF_TCS_LSH			25
#define SPI_CHxCONF_TCS_WID			2
#define SPI_CHxCONF_FFEW_LSH			27
#define SPI_CHxCONF_FFEW_WID			1
#define SPI_CHxCONF_FFER_LSH			28
#define SPI_CHxCONF_FFER_WID			1
#define SPI_CHxCONF_CLKG_LSH			29
#define SPI_CHxCONF_CLKG_WID			1

/* MCSPI_CHxSTAT */
#define SPI_CHxSTAT_RXS_LSH			0
#define SPI_CHxSTAT_RXS_WID			1
#define SPI_CHxSTAT_TXS_LSH			1
#define SPI_CHxSTAT_TXS_WID			1
#define SPI_CHxSTAT_EOT_LSH			2
#define SPI_CHxSTAT_EOT_WID			1
#define SPI_CHxSTAT_TXFFE_LSH			3
#define SPI_CHxSTAT_TXFFE_WID			1
#define SPI_CHxSTAT_TXFFF_LSH			4
#define SPI_CHxSTAT_TXFFF_WID			1
#define SPI_CHxSTAT_RXFFE_LSH			5
#define SPI_CHxSTAT_RXFFE_WID			1
#define SPI_CHxSTAT_RXFFF_LSH			6
#define SPI_CHxSTAT_RXFFF_WID			1

/* MCSPI_CHxCTRL */
#define SPI_CHxCTRL_EN_LSH			0
#define SPI_CHxCTRL_EN_WID			1
#define SPI_CHxCTRL_EXTCLK_LSH			8
#define SPI_CHxCTRL_EXTCLK_WID			8

/* MCSPI_TXx */
#define SPI_TXx_TDATA_LSH			0
#define SPI_TXx_TDATA_WID			32

/* MCSPI_RXx */
#define SPI_RXx_RDATA_LSH			0
#define SPI_RXx_RDATA_WID			32

/* MCSPI_XFERLEVEL */
#define SPI_AEL_LSH				0
#define SPI_AEL_WIP				6
#define SPI_AFL_LSH				8
#define SPI_AFL_WID				6
#define SPI_WCNT_LSH				16
#define SPI_WCNT_WID				16

/*
 * Mode CPOL CPHA
 * 0     0     0
 * 1     0     1
 * 2     1     0
 * 3     1     1
 */

typedef enum {
	SPI_MODE0 = 0,
	SPI_MODE1,
	SPI_MODE2,
	SPI_MODE3,
} SPIMODE;

#endif
/* __SPI__H__ */
